@@ -4,7 +4,7 @@
 // (c) 1995-2014 by Artsoft Entertainment
 //     		    Holger Schemel
 //		    info@artsoft.org
-//		    http://www.artsoft.org/
+//		    https://www.artsoft.org/
 // ----------------------------------------------------------------------------
 // anim.c
 // ============================================================================
@@ -108,6 +108,7 @@ struct GlobalAnimPartControlInfo
   int step_xoffset, step_yoffset;
 
   unsigned int initial_anim_sync_frame;
+  unsigned int anim_random_frame;
   unsigned int step_delay, step_delay_value;
 
   int init_delay_counter;
@@ -177,10 +178,13 @@ struct GameModeAnimClass
   { GAME_MODE_TITLE_3,			ANIM_CLASS_TITLE		},
   { GAME_MODE_TITLE_4,			ANIM_CLASS_TITLE		},
   { GAME_MODE_TITLE_5,			ANIM_CLASS_TITLE		},
+  { GAME_MODE_NAMES, 			ANIM_CLASS_TOONS_MENU_SUBMENU	},
   { GAME_MODE_LEVELS, 			ANIM_CLASS_TOONS_MENU_SUBMENU	},
   { GAME_MODE_LEVELNR,			ANIM_CLASS_TOONS_MENU_SUBMENU	},
   { GAME_MODE_INFO,			ANIM_CLASS_TOONS_MENU_SUBMENU	},
   { GAME_MODE_SETUP,			ANIM_CLASS_TOONS_MENU_SUBMENU	},
+  { GAME_MODE_PSEUDO_NAMESONLY,		ANIM_CLASS_TOONS_MENU_SUBMENU	},
+  { GAME_MODE_PSEUDO_TYPENAMES,		ANIM_CLASS_TOONS_MENU_SUBMENU	},
   { GAME_MODE_PSEUDO_MAINONLY,		ANIM_CLASS_TOONS_MENU_MAIN	},
   { GAME_MODE_PSEUDO_TYPENAME,		ANIM_CLASS_TOONS_MENU_MAIN	},
   { GAME_MODE_PSEUDO_SCORESOLD,		ANIM_CLASS_TOONS_SCORES		},
@@ -412,6 +416,7 @@ static void InitToonControls(void)
     part->control_info.y = ARG_UNDEFINED_VALUE;
 
     part->initial_anim_sync_frame = 0;
+    part->anim_random_frame = -1;
 
     part->step_delay = 0;
     part->step_delay_value = graphic_info[control].step_delay;
@@ -492,13 +497,15 @@ static void InitGlobalAnimControls(void)
 	  continue;
 
 #if 0
-	printf("::: mode == %d, anim = %d, part = %d [%d, %d, %d] [%d]\n",
-	       m, a, p, mode_nr, anim_nr, part_nr, control);
+	Debug("anim:InitGlobalAnimControls",
+	      "mode == %d, anim = %d, part = %d [%d, %d, %d] [%d]",
+	      m, a, p, mode_nr, anim_nr, part_nr, control);
 #endif
 
 #if 0
-	printf("::: mode == %d, anim = %d, part = %d [%d, %d, %d] [%d]\n",
-	       m, a, p, mode_nr, anim_nr, part_nr, sound);
+	Debug("anim:InitGlobalAnimControls",
+	      "mode == %d, anim = %d, part = %d [%d, %d, %d] [%d]",
+	      m, a, p, mode_nr, anim_nr, part_nr, sound);
 #endif
 
 	part->old_nr = p;
@@ -516,6 +523,7 @@ static void InitGlobalAnimControls(void)
 	part->control_info = graphic_info[control];
 
 	part->initial_anim_sync_frame = 0;
+	part->anim_random_frame = -1;
 
 	part->step_delay = 0;
 	part->step_delay_value = graphic_info[control].step_delay;
@@ -637,9 +645,11 @@ static void DrawGlobalAnimationsExt(int drawing_target, int drawing_stage)
     if (drawing_target == DRAW_TO_FADE_TARGET)
       after_fading = TRUE;
 
-    // special case: changing from/to this screen is done without fading
-    if (global.anim_status == GAME_MODE_PSEUDO_TYPENAME ||
-	anim_status_last   == GAME_MODE_PSEUDO_TYPENAME)
+    // special case: changing from/to these screens is done without fading
+    if (global.anim_status == GAME_MODE_PSEUDO_TYPENAME  ||
+	global.anim_status == GAME_MODE_PSEUDO_TYPENAMES ||
+	anim_status_last   == GAME_MODE_PSEUDO_TYPENAME  ||
+	anim_status_last   == GAME_MODE_PSEUDO_TYPENAMES)
       after_fading = TRUE;
 
     // ---------- part 1 ------------------------------------------------------
@@ -767,6 +777,7 @@ static void DrawGlobalAnimationsExt(int drawing_target, int drawing_stage)
 	  (g->draw_masked ? BlitBitmapMasked : BlitBitmap);
 	void (*blit_screen)(Bitmap *, int, int, int, int, int, int) =
 	  (g->draw_masked ? BlitToScreenMasked : BlitToScreen);
+	int last_anim_random_frame = gfx.anim_random_frame;
 
 	if (!(part->state & ANIM_STATE_RUNNING))
 	  continue;
@@ -799,10 +810,21 @@ static void DrawGlobalAnimationsExt(int drawing_target, int drawing_stage)
 	dst_y += part->viewport_y;
 
 	sync_frame = anim_sync_frame - part->initial_anim_sync_frame;
+
+	// re-initialize random animation frame after animation delay
+	if (g->anim_mode == ANIM_RANDOM &&
+	    sync_frame % g->anim_delay == 0 &&
+	    sync_frame > 0)
+	  part->anim_random_frame = GetSimpleRandom(g->anim_frames);
+
+	gfx.anim_random_frame = part->anim_random_frame;
+
 	frame = getAnimationFrame(g->anim_frames, g->anim_delay,
 				  g->anim_mode, g->anim_start_frame,
 				  sync_frame);
 
+	gfx.anim_random_frame = last_anim_random_frame;
+
 	getFixedGraphicSource(part->graphic, frame, &src_bitmap,
 			      &src_x, &src_y);
 
@@ -848,6 +870,8 @@ void DrawGlobalAnimations(int drawing_target, int drawing_stage)
     ResetGlobalAnim_Clicked();
   }
 
+  DrawEnvelopeRequestToScreen(drawing_target, drawing_stage);
+
   if (gfx.cursor_mode_override != last_cursor_mode_override)
     SetMouseCursor(gfx.cursor_mode);
 }
@@ -966,7 +990,7 @@ static void PlayGlobalAnimSound(struct GlobalAnimPartControlInfo *part)
     PlaySound(sound);
 
 #if 0
-  printf("::: PLAY SOUND %d.%d.%d: %d\n",
+  Debug("anim:PlayGlobalAnimSound", "PLAY SOUND %d.%d.%d: %d",
 	 part->anim_nr, part->nr, part->mode_nr, sound);
 #endif
 }
@@ -981,8 +1005,8 @@ static void StopGlobalAnimSound(struct GlobalAnimPartControlInfo *part)
   StopSound(sound);
 
 #if 0
-  printf("::: STOP SOUND %d.%d.%d: %d\n",
-	 part->anim_nr, part->nr, part->mode_nr, sound);
+  Debug("anim:StopGlobalAnimSound", "STOP SOUND %d.%d.%d: %d",
+	part->anim_nr, part->nr, part->mode_nr, sound);
 #endif
 }
 
@@ -1002,8 +1026,8 @@ static void PlayGlobalAnimMusic(struct GlobalAnimPartControlInfo *part)
     PlayMusic(music);
 
 #if 0
-  printf("::: PLAY MUSIC %d.%d.%d: %d\n",
-	 part->anim_nr, part->nr, part->mode_nr, music);
+  Debug("anim:PlayGlobalAnimMusic", "PLAY MUSIC %d.%d.%d: %d",
+	part->anim_nr, part->nr, part->mode_nr, music);
 #endif
 }
 
@@ -1017,8 +1041,8 @@ static void StopGlobalAnimMusic(struct GlobalAnimPartControlInfo *part)
   StopMusic();
 
 #if 0
-  printf("::: STOP MUSIC %d.%d.%d: %d\n",
-	 part->anim_nr, part->nr, part->mode_nr, music);
+  Debug("anim:StopGlobalAnimMusic", "STOP MUSIC %d.%d.%d: %d",
+	part->anim_nr, part->nr, part->mode_nr, music);
 #endif
 }
 
@@ -1131,6 +1155,11 @@ static boolean isClickedPart(struct GlobalAnimPartControlInfo *part,
   return TRUE;
 }
 
+static boolean clickBlocked(struct GlobalAnimPartControlInfo *part)
+{
+  return (part->control_info.style & STYLE_BLOCK ? TRUE : FALSE);
+}
+
 static boolean clickConsumed(struct GlobalAnimPartControlInfo *part)
 {
   return (part->control_info.style & STYLE_PASSTHROUGH ? FALSE : TRUE);
@@ -1170,13 +1199,15 @@ static void InitGlobalAnim_Triggered(struct GlobalAnimPartControlInfo *part,
 	*click_consumed |= clickConsumed(part); 	// click was on "part"!
 
 #if DEBUG_ANIM_EVENTS
-	printf("::: => %d.%d TRIGGERED BY %s OF %d.%d\n",
-	       part2->old_anim_nr + 1, part2->old_nr + 1, info_text,
-	       part->old_anim_nr + 1, part->old_nr + 1);
+	Debug("anim:InitGlobalAnim_Triggered",
+	      "%d.%d TRIGGERED BY %s OF %d.%d",
+	      part2->old_anim_nr + 1, part2->old_nr + 1, info_text,
+	      part->old_anim_nr + 1, part->old_nr + 1);
 #endif
 #if 0
-	printf("::: %d.%d TRIGGER CLICKED [%d]\n", anim2_nr, part2_nr,
-	       part2->control_info.anim_event_action);
+	Debug("anim:InitGlobalAnim_Triggered",
+	      "%d.%d TRIGGER CLICKED [%d]", anim2_nr, part2_nr,
+	      part2->control_info.anim_event_action);
 #endif
 
 	// after executing event action, ignore any further actions
@@ -1187,14 +1218,15 @@ static void InitGlobalAnim_Triggered(struct GlobalAnimPartControlInfo *part,
 #if 0
       struct GraphicInfo *c = &part2->control_info;
 
-      printf("::: - %d.%d: 0x%08x, 0x%08x [0x%08x]",
-	     anim2_nr, part2_nr, c->init_event, c->anim_event, mask);
-
       if (isClickablePart(part2, mask))
-	printf(" <--- TRIGGERED BY %d.%d",
-	       anim_nr, part_nr);
-
-      printf("\n");
+	Debug("anim:InitGlobalAnim_Triggered",
+	      "%d.%d: 0x%08x, 0x%08x [0x%08x] <--- TRIGGERED BY %d.%d",
+	      anim2_nr, part2_nr, c->init_event, c->anim_event, mask,
+	      anim_nr, part_nr);
+      else
+	Debug("anim:InitGlobalAnim_Triggered",
+	      "%d.%d: 0x%08x, 0x%08x [0x%08x]",
+	      anim2_nr, part2_nr, c->init_event, c->anim_event, mask);
 #endif
     }
   }
@@ -1204,7 +1236,8 @@ static void HandleGlobalAnimDelay(struct GlobalAnimPartControlInfo *part,
 				  int delay_type, char *info_text)
 {
 #if DEBUG_ANIM_DELAY
-  printf("::: %d.%d %s\n", part->old_anim_nr + 1, part->old_nr + 1, info_text);
+  Debug("anim:HandleGlobalAnimDelay", "%d.%d %s",
+	part->old_anim_nr + 1, part->old_nr + 1, info_text);
 #endif
 
   DoGlobalAnim_DelayAction(part, delay_type);
@@ -1214,7 +1247,8 @@ static void HandleGlobalAnimEvent(struct GlobalAnimPartControlInfo *part,
 				  int event_value, char *info_text)
 {
 #if DEBUG_ANIM_EVENTS
-  printf("::: %d.%d %s\n", part->old_anim_nr + 1, part->old_nr + 1, info_text);
+  Debug("anim:HandleGlobalAnimEvent", "%d.%d %s",
+	part->old_anim_nr + 1, part->old_nr + 1, info_text);
 #endif
 
   boolean click_consumed = FALSE;
@@ -1263,6 +1297,10 @@ static int HandleGlobalAnim_Part(struct GlobalAnimPartControlInfo *part,
     part->initial_anim_sync_frame =
       (g->anim_global_sync ? 0 : anim_sync_frame + part->init_delay_counter);
 
+    // do not re-initialize random animation frame after fade-in
+    if (part->anim_random_frame == -1)
+      part->anim_random_frame = GetSimpleRandom(g->anim_frames);
+
     if (c->direction & MV_HORIZONTAL)
     {
       int pos_bottom = part->viewport_height - g->height;
@@ -1484,8 +1522,8 @@ static int HandleGlobalAnim_Part(struct GlobalAnimPartControlInfo *part,
     static unsigned int last_counter = -1;
     unsigned int counter = Counter();
 
-    printf("::: NEXT ANIM PART [%d, %d]\n",
-	   anim_sync_frame, counter - last_counter);
+    Debug("anim:HandleGlobalAnim_Part", "NEXT ANIM PART [%d, %d]",
+	  anim_sync_frame, counter - last_counter);
 
     last_counter = counter;
   }
@@ -1510,21 +1548,22 @@ static void HandleGlobalAnim_Main(struct GlobalAnimMainControlInfo *anim,
   int i;
 
 #if 0
-  printf("::: HandleGlobalAnim_Main: %d, %d => %d\n",
+  Debug("anim:HandleGlobalAnim_Main", "%d, %d => %d",
 	 anim->mode_nr, anim->nr, anim->num_parts);
-  printf("::: %d, %d, %d\n", global.num_toons, setup.toons, num_toons);
+  Debug("anim:HandleGlobalAnim_Main",
+	"%d, %d, %d", global.num_toons, setup.toons, num_toons);
 #endif
 
 #if 0
-  printf("::: %s(%d): %d, %d, %d [%d]\n",
-	 (action == ANIM_START ? "ANIM_START" :
-	  action == ANIM_CONTINUE ? "ANIM_CONTINUE" :
-	  action == ANIM_STOP ? "ANIM_STOP" : "(should not happen)"),
-	 anim->nr,
-	 anim->state & ANIM_STATE_RESTART,
-	 anim->state & ANIM_STATE_WAITING,
-	 anim->state & ANIM_STATE_RUNNING,
-	 anim->num_parts);
+  Debug("anim:HandleGlobalAnim_Main", "%s(%d): %d, %d, %d [%d]",
+	(action == ANIM_START ? "ANIM_START" :
+	 action == ANIM_CONTINUE ? "ANIM_CONTINUE" :
+	 action == ANIM_STOP ? "ANIM_STOP" : "(should not happen)"),
+	anim->nr,
+	anim->state & ANIM_STATE_RESTART,
+	anim->state & ANIM_STATE_WAITING,
+	anim->state & ANIM_STATE_RUNNING,
+	anim->num_parts);
 #endif
 
   switch (action)
@@ -1560,8 +1599,8 @@ static void HandleGlobalAnim_Main(struct GlobalAnimMainControlInfo *anim,
   if (c->anim_mode & ANIM_ALL || anim->num_parts == 0)
   {
 #if 0
-    printf("::: HandleGlobalAnim_Main: %d, %d => %d\n",
-	   anim->mode_nr, anim->nr, num_parts);
+    Debug("anim:HandleGlobalAnim_Main", "%d, %d => %d",
+	  anim->mode_nr, anim->nr, num_parts);
 #endif
 
     for (i = 0; i < num_parts; i++)
@@ -1658,8 +1697,7 @@ static void HandleGlobalAnim_Mode(struct GlobalAnimControlInfo *ctrl, int action
   int i;
 
 #if 0
-  printf("::: HandleGlobalAnim_Mode: %d => %d\n",
-	 ctrl->nr, ctrl->num_anims);
+  Debug("anim:HandleGlobalAnim_Mode", "%d => %d", ctrl->nr, ctrl->num_anims);
 #endif
 
   for (i = 0; i < ctrl->num_anims; i++)
@@ -1669,7 +1707,7 @@ static void HandleGlobalAnim_Mode(struct GlobalAnimControlInfo *ctrl, int action
 static void HandleGlobalAnim(int action, int game_mode)
 {
 #if 0
-  printf("::: HandleGlobalAnim [mode == %d]\n", game_status);
+  Debug("anim:HandleGlobalAnim", "mode == %d", game_status);
 #endif
 
   HandleGlobalAnim_Mode(&global_anim_ctrl[game_mode], action);
@@ -1680,7 +1718,7 @@ static void DoAnimationExt(void)
   int i;
 
 #if 0
-  printf("::: DoAnimation [%d, %d]\n", anim_sync_frame, Counter());
+  Debug("anim:DoAnimationExt", "%d, %d", anim_sync_frame, Counter());
 #endif
 
   // global animations now synchronized with frame delay of screen update
@@ -1804,8 +1842,8 @@ static boolean InitGlobalAnim_Clicked(int mx, int my, int clicked_event)
 	    isClickablePart(part, ANIM_EVENT_ANY))
 	{
 #if DEBUG_ANIM_EVENTS
-	  printf("::: => %d.%d TRIGGERED BY ANY\n",
-		 part->old_anim_nr + 1, part->old_nr + 1);
+	  Debug("anim:InitGlobalAnim_Clicked", "%d.%d TRIGGERED BY ANY",
+		part->old_anim_nr + 1, part->old_nr + 1);
 #endif
 
 	  anything_clicked = part->clicked = TRUE;
@@ -1817,8 +1855,8 @@ static boolean InitGlobalAnim_Clicked(int mx, int my, int clicked_event)
 	    isClickablePart(part, ANIM_EVENT_UNCLICK_ANY))
 	{
 #if DEBUG_ANIM_EVENTS
-	  printf("::: => %d.%d TRIGGERED BY UNCLICK:ANY\n",
-		 part->old_anim_nr + 1, part->old_nr + 1);
+	  Debug("anim:InitGlobalAnim_Clicked", "%d.%d TRIGGERED BY UNCLICK:ANY",
+		part->old_anim_nr + 1, part->old_nr + 1);
 #endif
 
 	  anything_clicked = part->clicked = TRUE;
@@ -1833,8 +1871,8 @@ static boolean InitGlobalAnim_Clicked(int mx, int my, int clicked_event)
 	    isClickedPart(part, mx, my, TRUE))
 	{
 #if 0
-	  printf("::: %d.%d CLICKED [%d]\n", anim_nr, part_nr,
-		 part->control_info.anim_event_action);
+	  Debug("anim:InitGlobalAnim_Clicked", "%d.%d CLICKED [%d]",
+		anim_nr, part_nr, part->control_info.anim_event_action);
 #endif
 
 	  // after executing event action, ignore any further actions
@@ -1847,14 +1885,17 @@ static boolean InitGlobalAnim_Clicked(int mx, int my, int clicked_event)
 	  if (isClickablePart(part, ANIM_EVENT_SELF))
 	  {
 #if DEBUG_ANIM_EVENTS
-	    printf("::: => %d.%d TRIGGERED BY SELF\n",
-		   part->old_anim_nr + 1, part->old_nr + 1);
+	    Debug("anim:InitGlobalAnim_Clicked", "%d.%d TRIGGERED BY SELF",
+		  part->old_anim_nr + 1, part->old_nr + 1);
 #endif
 
 	    anything_clicked = part->clicked = TRUE;
 	    click_consumed |= clickConsumed(part);
 	  }
 
+	  // determine if mouse clicks should be blocked by this animation
+	  click_consumed |= clickBlocked(part);
+
 	  // check if this click is defined to trigger other animations
 	  InitGlobalAnim_Triggered(part, &click_consumed, &any_event_action,
 				   ANIM_EVENT_CLICK, "CLICK");
