@@ -4,7 +4,7 @@
 // (c) 1995-2014 by Artsoft Entertainment
 //     		    Holger Schemel
 //		    info@artsoft.org
-//		    http://www.artsoft.org/
+//		    https://www.artsoft.org/
 // ----------------------------------------------------------------------------
 // files.h
 // ============================================================================
@@ -33,6 +33,8 @@ char *getDefaultLevelFilename(int);
 char *getLocalLevelTemplateFilename(void);
 char *getGlobalLevelTemplateFilename(void);
 
+int getMappedElement(int);
+
 void LoadLevelFromFilename(struct LevelInfo *, char *);
 void LoadLevel(int);
 void LoadLevelTemplate(int);
@@ -50,6 +52,7 @@ void CopyNativeLevel_Native_to_RND(struct LevelInfo *);
 void LoadTapeFromFilename(char *);
 void LoadTape(int);
 void LoadSolutionTape(int);
+void SaveTapeToFilename(char *);
 void SaveTape(int);
 void DumpTape(struct TapeInfo *);
 boolean SaveTapeChecked(int);
@@ -58,6 +61,8 @@ boolean SaveTapeChecked_LevelSolved(int);
 void LoadScore(int);
 void SaveScore(int);
 
+void LoadUserNames(void);
+
 void LoadSetupFromFilename(char *);
 void LoadSetup(void);
 void SaveSetup(void);
@@ -71,6 +76,7 @@ void SaveSetup_EditorCascade(void);
 void SaveSetup_AddGameControllerMapping(char *);
 
 void setHideSetupEntry(void *);
+void removeHideSetupEntry(void *);
 boolean hideSetupEntry(void *);
 
 void LoadCustomElementDescriptions(void);
