@@ -4,7 +4,7 @@
 // (c) 1995-2014 by Artsoft Entertainment
 //     		    Holger Schemel
 //		    info@artsoft.org
-//		    http://www.artsoft.org/
+//		    https://www.artsoft.org/
 // ----------------------------------------------------------------------------
 // game.h
 // ============================================================================
@@ -178,6 +178,10 @@ struct GameInfo
   boolean max_num_changes_per_frame;
   boolean use_reverse_scan_direction;
 
+  // flags to indicate which game actions are used in this game
+  boolean use_key_actions;
+  boolean use_mouse_actions;
+
   // variable within running game
   int yamyam_content_nr;
   boolean robot_wheel_active;
@@ -215,7 +219,7 @@ struct GameInfo
   // values for the new EMC elements
   int lenses_time_left;
   int magnify_time_left;
-  boolean ball_state;
+  boolean ball_active;
   int ball_content_nr;
 
   // values for player idle animation (no effect on engine)
@@ -232,11 +236,16 @@ struct GameInfo
 
   // values for special request dialog control
   boolean request_active;
+  boolean request_active_or_moving;
 
   // values for special game control
   int centered_player_nr;
   int centered_player_nr_next;
   boolean set_centered_player;
+  boolean set_centered_player_wrap;
+
+  // values for single step mode control
+  boolean enter_single_step_mode;
 
   // values for random number generator initialization after snapshot
   unsigned int num_random_calls;
@@ -372,6 +381,8 @@ struct PlayerInfo
   int shield_normal_time_left;
   int shield_deadly_time_left;
 
+  int last_removed_element;
+
   int inventory_element[MAX_INVENTORY_SIZE];
   int inventory_infinite_element;
   int inventory_size;
@@ -390,7 +401,7 @@ int GetElementFromGroupElement(int);
 
 int getPlayerInventorySize(int);
 
-void DrawGameValue_Time(int);
+void UpdateGameDoorValues(void);
 void DrawGameDoorValues(void);
 
 void UpdateAndDisplayGameControlValues(void);
