@@ -4,7 +4,7 @@
 // (c) 1994-2017 by Artsoft Entertainment
 //     		    Holger Schemel
 //		    info@artsoft.org
-//		    http://www.artsoft.org/
+//		    https://www.artsoft.org/
 // ----------------------------------------------------------------------------
 // mm_main.h
 // ============================================================================
@@ -123,10 +123,10 @@
 				 IS_WALL_AMOEBA(e))
 
 #define CAN_MOVE(e)		((e) == EL_PACMAN)
-#define IS_FREE(x,y)            (Feld[x][y] == EL_EMPTY)
+#define IS_FREE(x,y)            (Tile[x][y] == EL_EMPTY)
 
 #define IS_MOVING(x,y)          (MovPos[x][y] != 0)
-#define IS_BLOCKED(x,y)         (Feld[x][y] == EL_BLOCKED)
+#define IS_BLOCKED(x,y)         (Tile[x][y] == EL_BLOCKED)
 #define IS_DRAWABLE(e)          ((e) < EL_BLOCKED)
 #define IS_NOT_DRAWABLE(e)      ((e) >= EL_BLOCKED)
 
@@ -194,7 +194,7 @@ extern boolean		level_editor_test_game;
 extern boolean		redraw[MAX_LEV_FIELDX][MAX_LEV_FIELDY];
 extern int		redraw_x1, redraw_y1;
 
-extern short		Feld[MAX_LEV_FIELDX][MAX_LEV_FIELDY];
+extern short		Tile[MAX_LEV_FIELDX][MAX_LEV_FIELDY];
 extern short		Ur[MAX_LEV_FIELDX][MAX_LEV_FIELDY];
 extern short		Hit[MAX_LEV_FIELDX][MAX_LEV_FIELDY];
 extern short		Box[MAX_LEV_FIELDX][MAX_LEV_FIELDY];
@@ -226,11 +226,8 @@ extern int		TimeFrames, TimePlayed, TimeLeft;
 
 extern struct LevelInfo_MM	native_mm_level;
 extern struct HiScore_MM	highscore[];
-extern struct TapeInfo		tape;
-extern struct SetupInfo		setup;
 extern struct GameInfo_MM	game_mm;
 extern struct LaserInfo		laser;
-extern struct GlobalInfo	global;
 
 extern short 		LX, LY, XS, YS, ELX, ELY;
 extern short 		CT, Ct;
