@@ -4,7 +4,7 @@
 // (c) 1995-2014 by Artsoft Entertainment
 //     		    Holger Schemel
 //		    info@artsoft.org
-//		    http://www.artsoft.org/
+//		    https://www.artsoft.org/
 // ----------------------------------------------------------------------------
 // misc.h
 // ============================================================================
@@ -34,23 +34,6 @@
 #define GetEngineRandom(max)		get_random_number(RANDOM_ENGINE, max)
 #define GetSimpleRandom(max)		get_random_number(RANDOM_SIMPLE, max)
 
-// values for Error()
-#define ERR_UNKNOWN			0
-#define ERR_DEBUG			(1 << 0)
-#define ERR_INFO			(1 << 1)
-#define ERR_INFO_LINE			(1 << 2)
-#define ERR_WARN			(1 << 3)
-#define ERR_EXIT			(1 << 4)
-#define ERR_HELP			(1 << 5)
-#define ERR_SOUND_SERVER		(1 << 6)
-#define ERR_NETWORK_SERVER		(1 << 7)
-#define ERR_NETWORK_CLIENT		(1 << 8)
-#define ERR_FROM_SERVER			(ERR_SOUND_SERVER | ERR_NETWORK_SERVER)
-#define ERR_EXIT_HELP			(ERR_EXIT | ERR_HELP)
-#define ERR_EXIT_SOUND_SERVER		(ERR_EXIT | ERR_SOUND_SERVER)
-#define ERR_EXIT_NETWORK_SERVER		(ERR_EXIT | ERR_NETWORK_SERVER)
-#define ERR_EXIT_NETWORK_CLIENT		(ERR_EXIT | ERR_NETWORK_CLIENT)
-
 // values for getFile...() and putFile...()
 #define BYTE_ORDER_BIG_ENDIAN		0
 #define BYTE_ORDER_LITTLE_ENDIAN	1
@@ -123,6 +106,14 @@ void PrintNoLog(char *, ...);
 void PrintLine(char *, int);
 void PrintLineWithPrefix(char *, char *, int);
 
+void DebugContinued(char *, char *, ...);
+void Debug(char *, char *, ...);
+void Info(char *, ...);
+void Warn(char *, ...);
+void Error(char *, ...);
+void Fail(char *, ...);
+void FailWithHelp(char *, ...);
+
 char *int2str(int, int);
 char *i_to_a(unsigned int);
 int log_2(unsigned int);
@@ -144,8 +135,19 @@ void SkipUntilDelayReached(unsigned int *, unsigned int, int *, int);
 unsigned int init_random_number(int, int);
 unsigned int get_random_number(int, int);
 
+#if defined(PLATFORM_UNIX)
+char *getUnixLoginName(void);
+char *getUnixRealName(void);
+char *getUnixHomeDir(void);
+#endif
+
 char *getLoginName(void);
 char *getRealName(void);
+char *getFixedUserName(char *);
+char *getDefaultUserName(int);
+
+char *getTimestampFromEpoch(time_t);
+char *getCurrentTimestamp(void);
 
 time_t getFileTimestampEpochSeconds(char *);
 
@@ -178,10 +180,6 @@ void GetOptions(int, char **,
 		void (*print_usage_function)(void),
 		void (*print_version_function)(void));
 
-void SetError(char *, ...);
-char *GetError(void);
-void Error(int, char *, ...);
-
 void *checked_malloc(unsigned int);
 void *checked_calloc(unsigned int);
 void *checked_realloc(void *, unsigned int);
