@@ -4,7 +4,7 @@
 // (c) 1995-2014 by Artsoft Entertainment
 //     		    Holger Schemel
 //		    info@artsoft.org
-//		    http://www.artsoft.org/
+//		    https://www.artsoft.org/
 // ----------------------------------------------------------------------------
 // text.c
 // ============================================================================
@@ -169,7 +169,7 @@ void DrawTextF(int x, int y, int font_nr, char *format, ...)
   va_end(ap);
 
   if (strlen(buffer) > MAX_OUTPUT_LINESIZE)
-    Error(ERR_EXIT, "string too long in DrawTextF() -- aborting");
+    Fail("string too long in DrawTextF() -- aborting");
 
   DrawText(gfx.sx + x, gfx.sy + y, buffer, font_nr);
 }
@@ -184,7 +184,7 @@ void DrawTextFCentered(int y, int font_nr, char *format, ...)
   va_end(ap);
 
   if (strlen(buffer) > MAX_OUTPUT_LINESIZE)
-    Error(ERR_EXIT, "string too long in DrawTextFCentered() -- aborting");
+    Fail("string too long in DrawTextFCentered() -- aborting");
 
   DrawText(gfx.sx + (gfx.sxsize - getTextWidth(buffer, font_nr)) / 2,
 	   gfx.sy + y, buffer, font_nr);
@@ -639,7 +639,7 @@ int DrawTextBufferVA(int x, int y, char *format, va_list ap, int font_nr,
   int text_length = vsnprintf(text_buffer, MAX_OUTPUT_LINESIZE, format, ap);
 
   if (text_length >= MAX_OUTPUT_LINESIZE)
-    Error(ERR_WARN, "string too long in DrawTextBufferVA() -- truncated");
+    Warn("string too long in DrawTextBufferVA() -- truncated");
 
   int num_lines_printed = DrawTextBuffer(x, y, text_buffer, font_nr,
 					 line_length, cut_length, max_lines,
