@@ -4,7 +4,7 @@
 // (c) 1995-2014 by Artsoft Entertainment
 //     		    Holger Schemel
 //		    info@artsoft.org
-//		    http://www.artsoft.org/
+//		    https://www.artsoft.org/
 // ----------------------------------------------------------------------------
 // tape.h
 // ============================================================================
@@ -19,13 +19,25 @@
 #define	TAPE_TOGGLE_PLAY_PAUSE	(1 << 1)
 
 // values for tape properties
-#define MAX_TAPE_LEN		(1000 * FRAMES_PER_SECOND) // max.time x fps
+#define MAX_TAPE_LEN		(10000 * FRAMES_PER_SECOND) // max.time x fps
 
-// values for tape mouse actions
-#define TAPE_ACTION_LX		0
-#define TAPE_ACTION_LY		1
-#define TAPE_ACTION_BUTTON	2
-#define TAPE_ACTION_UNUSED	3
+// values for tape action array positions
+#define TAPE_ACTION_LX		(MAX_PLAYERS + 0)
+#define TAPE_ACTION_LY		(MAX_PLAYERS + 1)
+#define TAPE_ACTION_BUTTON	(MAX_PLAYERS + 2)
+
+#define MAX_TAPE_ACTIONS	(MAX_PLAYERS + 3)
+
+// values for tape actions stored in tape file
+#define TAPE_USE_KEY_ACTIONS_ONLY	0
+#define TAPE_USE_MOUSE_ACTIONS_ONLY	1
+#define TAPE_USE_KEY_AND_MOUSE_ACTIONS	2
+
+#define TAPE_ACTIONS_DEFAULT		TAPE_USE_KEY_ACTIONS_ONLY
+
+// values for tape properties stored in tape file
+#define TAPE_PROPERTY_NONE		0
+#define TAPE_PROPERTY_EM_RANDOM_BUG	(1 << 0)
 
 // some positions in the video tape control window
 #define VIDEO_DISPLAY1_XPOS	5
@@ -182,6 +194,8 @@ struct TapeInfo
   boolean deactivate_display;
   boolean auto_play;
   boolean auto_play_level_solved;
+  boolean auto_play_level_fixed;
+  boolean auto_play_level_not_fixable;
   boolean quick_resume;
   boolean single_step;
   boolean changed;
@@ -190,11 +204,20 @@ struct TapeInfo
   int centered_player_nr_next;
   boolean set_centered_player;
 
-  boolean use_mouse;
+  // flags to indicate which game actions are stored in this tape
+  boolean use_key_actions;
+  boolean use_mouse_actions;
+
+  // bits to indicate which tape properties are stored in this tape
+  byte property_bits;
+
+  // visible playfield size when recording this tape (for team mode)
+  int scr_fieldx;
+  int scr_fieldy;
 
   struct
   {
-    byte action[MAX_PLAYERS];
+    byte action[MAX_TAPE_ACTIONS];
     byte delay;
   } pos[MAX_TAPE_LEN];
 
@@ -238,7 +261,11 @@ boolean hasSolutionTape(void);
 boolean InsertSolutionTape(void);
 boolean PlaySolutionTape(void);
 
-void AutoPlayTape(void);
+void UndoTape(void);
+void FixTape_ForceSinglePlayer(void);
+
+void AutoPlayTapes(void);
+void PatchTapes(void);
 
 void CreateTapeButtons(void);
 void FreeTapeButtons(void);
